%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CalculateFits_core : Utility Function
% 
% Reference: Fetal/Juvenile Tumor Initiation (2010)
% Authors: Kini, L.G., Herrero-Jimenez P., Guitterez Jr E., Hensle, D.,
%    Kogel J., Kusko R., Rexer, K., Kurzweil, R., Gostjeva, E., 
%    Morgenthaler, S., and Thilly WG.
% Contact: Lohith Kini <lkini (AT) mit DOT edu>
% Version: 5.0
% Last modified: March 22, 2010
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ Fseries,sfseries,museries,Riseries,RAseries, top_fits, CAL] = CalculateFits_core( nF,Fstart,Fend,nf,sfstart,sfend,...
    nmu,mustart,muend,nRi,Ristart,Riend,nRA,RAstart,RAend, ...
    amax, n, m, INC, num_fits)
%CALCULATEFITS_CORE Computes best fits for CancerFit through manual input
%   of parameter ranges and iterations
%
%  Input variables:
%   nF = Number of iterations for F
%   Fstart = Starting point for F search range
%   Fend = Ending point for F search range
%
%   nf = Number of iterations for f
%   sfstart = Starting point for f search range
%   sfend = Ending point for f search range
% 
%   nmu = Number of iterations for mu
%   mustart = Starting point for mu search range
%   muend = Ending point for mu search range
%
%   nRi = Number of iterations for R_init
%   Ristart = Starting point for R_init search range
%   Riend = Ending point for R_init search range
%
%   nRA = Number of iterations for R_prom
%   RAstart = Starting point for R_prom search range
%   RAend = Ending point for R_prom search range
%
%   amax,n,m = Model parameters as defined in paper
%   INC = Incidence per capital for birth cohort h for 5-year adult age groups
%       17.5 to 102.5
%   num_fits = Number of best fits to be calculated
%
%  Output variables:
%   Fseries = Array of search iterations for F
%   sfseries = Array of search iterations for f
%   museries = Array of search iterations for mu
%   Riseries = Array of search iterations for R_init
%   RAseries = Array of search iterations for R_prom
%
%   top_fits = Best fits array of size num_fits x 6
%   CAL = CAL(g) for each best fit; size of array is num_fits x 51

disp('Initializing CancerFit Fit calculation engine ... ')
start_time = cputime;

disp('Initializing Fit Parameters ...')
if(Fstart == 0)
    Fstart = 1E-4;
end

if(sfstart == 0)
    sfstart = 1E-4;
end



disp('Computing Fits ...')

if(nF == 1)
    Fseries = Fstart:1:Fstart;
else
    Fseries = Fstart:(Fend-Fstart)/(nF-1):Fend;
end

if(nf == 1)
    sfseries = sfstart:1:sfstart;
else
    sfseries = sfstart:(sfend-sfstart)/(nf-1):sfend;
end

if(nmu == 1)
    museries = mustart:1:mustart;
else
    museries = mustart:(muend-mustart)/(nmu-1):muend;
end

[int_Vobs_large,F,f,mu] = ndgrid(0:1:50,Fseries,sfseries,museries);
[tmpF,tmpf,tmpmu] = ndgrid(Fseries,sfseries,museries);

[G,A] = meshgrid(0:1:50,amax-floor(amax):1:amax);
age_x = [17.5:5:102.5];
age_x_m = [0:5:102.5-17.5];

Pobs_large = zeros(51,nF,nf,nmu);
CAL_interp = zeros(18,nF,nf,nmu);

INC_tmp = INC(8:end)*1E-5;
INC_large = repmat(INC_tmp', [1 nF nf nmu]);

error_term = zeros(nF,nf,nmu);
error_sq_term = zeros(nF,nf,nmu);
error_reshaped = zeros(nF*nf*nmu,1);
top_fits = 1E10*ones(num_fits,6);
merged_fits = 1E10*ones(2*num_fits,6);

x = 0:1:50;

h=waitbar(0,'Fit Calculation Progress ... 0%');
set(h,'windowstyle','modal');

if (nRi == 1)
    Riend = Ristart;
end
if (nRA == 1)
    RAend = RAstart;
end

if(abs(log10(Ristart/Riend)) > 3)
    for i = 1:nRi
        Riseries(i) = 10^((log10(Riend)-log10(Ristart))/(nRi-1)*(i-1) + log10(Ristart));
    end
else
    Riseries = Ristart:(Riend-Ristart)/(nRi-1):Riend;
end

if(abs(log10(RAstart/RAend)) > 3)
    for i = 1:nRA
        RAseries(i) = 10^((log10(RAend)-log10(RAstart))/(nRA-1)*(i-1) + log10(RAstart));
    end
else
    RAseries = RAstart:(RAend-RAstart)/(nRA-1):RAend;
end



frac_counter = 0;
for Ri = Riseries
    pause(0.2)
    frac = frac_counter/nRi;
    frac_counter = frac_counter+1;
    waitbar(frac,h,strcat('Fit Calculation Progress ... ',num2str(floor(frac*100)),'%'));
    for RA = RAseries
        if(m >= 2)
            Vobs = n*Ri^n*RA^m* A.^(n-1) .* 2.^G .* (2*(G-A+1).^(m-1) - (G-A).^(m-1)) .* exp(- RA^m * (G-A).^(m-1) .* 2.^(G-A) );
        else
            Vobs = n*Ri^n*RA^m*log(2)* A.^(n-1) .* 2.^G .* exp(- RA^m * (G-A).^(m-1) .* 2.^(G-A) );
        end
        
        mask = zeros(floor(amax)+1,51);

        for g = 0:1:50
            for a = 1:1:min(floor(amax),g)+1
                mask(a,g+1) = 1;
            end
        end
        
        Vobs = Vobs .* mask;
        
        Vobs = sum(Vobs,1);
        Pobs = 1 - exp(-Vobs);        
        Pobs(find(Pobs == 0)) = 1E-100;
        Pobs_tmp = ones(51,1,1,1);
        Pobs_tmp(:,1,1,1) = Pobs;
        Pobs_large = repmat(Pobs_tmp,[1 nF nf nmu]);
        
        

        int_Vobs = cumsum(Vobs);
        int_Vobs_tmp = ones(51,1,1,1);
        int_Vobs_tmp(:,1,1,1) = int_Vobs(:);
        int_Vobs_large = repmat(int_Vobs_tmp,[1 nF nf nmu]);
        
             
        coeff = F  ./ (F + (1-F) .* exp(1/f .* int_Vobs_large) );       
        Pobs_large = Pobs_large .* coeff;
        
        is = 1;
        for mu_val = mustart:(muend-mustart)/(nmu-1):muend                   
            age = age_x_m*mu_val+amax;            
            y = Pobs_large(:,:,:,is);
            yi = interp1(x',y,age);
            CAL_interp(:,:,:,is) = yi;
            error_sq_term(:,:,is) = sum((log10(mu_val*CAL_interp(:,:,:,is) ./ INC_large(:,:,:,is))).^2,1);
            if(~isreal(error_sq_term))
                'break';
            end
            is = is + 1;
        end        
        
        error_term = (error_sq_term/18).^0.5;        
        error_reshaped = reshape(error_term, [nF*nf*nmu 1]);
        error_term = zeros(nF,nf,nmu);
        [error_reshaped,ix] = sort(error_reshaped);

        num_fits_lim = min(num_fits,nF*nf*nmu);
        top_fits_tmp(:,1) = error_reshaped(1:num_fits_lim,1);
        top_fits_tmp(:,2) = tmpF(ix(1:num_fits_lim));
        top_fits_tmp(:,3) = tmpf(ix(1:num_fits_lim));
        top_fits_tmp(:,4) = tmpmu(ix(1:num_fits_lim));
        top_fits_tmp(:,5) = Ri;
        top_fits_tmp(:,6) = RA;       
        
        merged_fits(1:num_fits_lim,:) = top_fits_tmp;
        merged_fits(num_fits_lim+1:num_fits_lim+num_fits,:) = top_fits;
        [sorted_merged_fits,ix] = sort(merged_fits,1);
        sorted_merged_fits(:,2:6) = merged_fits(ix(:,1),2:6);
        top_fits = sorted_merged_fits(1:num_fits,:);
    end
end
close(h);


CAL = zeros(num_fits,51);
for is = 1:num_fits
    CAL(is,:) = CalcCAL( amax,m,n, top_fits(is,2),top_fits(is,3),top_fits(is,4),top_fits(is,5),top_fits(is,6) );
end


disp('Elapsed time for FIT Calculation ...')
cputime - start_time


end
